package ninja.thefirearchmage.games.fourtykcalculator;

import lombok.*;

import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

/**
 * Assumes a D6
 */
@Getter @AllArgsConstructor
public class Reroll {
    private RerollStageType rerollStageType;
    private RerollType type;
    private final Set<Integer> rerollableValues;
    private Set<UnitTypeKeyword> validTargetTypes;

    public static Reroll rerollOnOnes(RerollStageType rerollStageType, Set<UnitTypeKeyword> validTargetTypes) {
        return new Reroll(rerollStageType, RerollType.ON_ONES, Set.of(1), validTargetTypes);
    }

    public static Reroll rerollFailures(RerollStageType rerollStageType, int targetHit,
                                        Set<UnitTypeKeyword> validTargetTypes) {
        var rerollableValues = IntStream.range(1, targetHit).boxed()
                .collect(Collectors.toSet());

        return new Reroll(rerollStageType, RerollType.ON_FAILURE, rerollableValues, validTargetTypes);
    }

    public static Reroll rerollOnNot(RerollStageType rerollStageType, Set<Integer> allowedValues,
                                     Set<UnitTypeKeyword> validTargetTypes) {
        var rerollableValues = IntStream.range(1,7).filter(i-> !allowedValues.contains(i))
                .boxed().collect(Collectors.toSet());

        return new Reroll(rerollStageType, RerollType.ANY, rerollableValues, validTargetTypes);
    }

    /**
     * From a D6 returns what proportion of rerollable values this reroll is.
     */
    public double getProbabilityNumerator() {
        return (double) rerollableValues.size() / 6;
    }

    public boolean appliesTo(UnitTypeKeyword target) {
        return this.validTargetTypes.contains(target);
    }
}
